@@ -1,4 +1,7 @@
-/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
+/* SPDX-License-Identifier: GPL-2.0
+ *
+ * Copyright (C) 2015-2018 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
+ */
 
 #ifndef _WG_COMPAT_H
 #define _WG_COMPAT_H
@@ -22,6 +25,9 @@
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 5, 0) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 4, 0)
 #define ISOPENSUSE42
 #endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 12, 0)
+#define ISOPENSUSE15
+#endif
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 10, 0)
@@ -53,14 +59,10 @@
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
+#include <linux/rcupdate.h>
+#ifndef RCU_LOCKDEP_WARN
 #define RCU_LOCKDEP_WARN(cond, message) rcu_lockdep_assert(!(cond), message)
 #endif
-
-#if ((LINUX_VERSION_CODE > KERNEL_VERSION(3, 19, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(3, 19, 6)) || \
-    (LINUX_VERSION_CODE < KERNEL_VERSION(3, 18, 12) && LINUX_VERSION_CODE > KERNEL_VERSION(3, 17, 0)) || \
-    (LINUX_VERSION_CODE < KERNEL_VERSION(3, 16, 8) && LINUX_VERSION_CODE >= KERNEL_VERSION(3, 15, 0)) || \
-    LINUX_VERSION_CODE < KERNEL_VERSION(3, 14, 40)) && !defined(ISRHEL7) && !defined(ISUBUNTU1404)
-#define dev_recursion_level() 0
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0) && !defined(ISRHEL7)
@@ -215,19 +217,43 @@ static inline u32 prandom_u32_max(u32 ep_ro)
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 10, 75) && !defined(ISRHEL7)
+#ifndef U8_MAX
 #define U8_MAX ((u8)~0U)
+#endif
+#ifndef S8_MAX
 #define S8_MAX ((s8)(U8_MAX >> 1))
+#endif
+#ifndef S8_MIN
 #define S8_MIN ((s8)(-S8_MAX - 1))
+#endif
+#ifndef U16_MAX
 #define U16_MAX ((u16)~0U)
+#endif
+#ifndef S16_MAX
 #define S16_MAX ((s16)(U16_MAX >> 1))
+#endif
+#ifndef S16_MIN
 #define S16_MIN ((s16)(-S16_MAX - 1))
+#endif
+#ifndef U32_MAX
 #define U32_MAX ((u32)~0U)
+#endif
+#ifndef S32_MAX
 #define S32_MAX ((s32)(U32_MAX >> 1))
+#endif
+#ifndef S32_MIN
 #define S32_MIN ((s32)(-S32_MAX - 1))
+#endif
+#ifndef U64_MAX
 #define U64_MAX ((u64)~0ULL)
+#endif
+#ifndef S64_MAX
 #define S64_MAX ((s64)(U64_MAX >> 1))
+#endif
+#ifndef S64_MIN
 #define S64_MIN ((s64)(-S64_MAX - 1))
 #endif
+#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 10, 60) && !defined(ISRHEL7)
 /* Making this static may very well invalidate its usefulness,
@@ -244,7 +270,7 @@ static const struct in6_addr our_in6addr_any = IN6ADDR_ANY_INIT;
 #define in6addr_any our_in6addr_any
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 2, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 2, 0) && !defined(ISOPENSUSE15)
 #include <linux/completion.h>
 #include <linux/random.h>
 #include <linux/errno.h>
@@ -287,7 +313,7 @@ static inline int wait_for_random_bytes(void)
 	return 0;
 }
 #endif
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0) && !defined(ISOPENSUSE15)
 static inline int get_random_bytes_wait(void *buf, int nbytes)
 {
 	int ret = wait_for_random_bytes();
@@ -404,7 +430,7 @@ static inline void kvfree_ours(const void *addr)
 #define priv_destructor destructor
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0) && !defined(ISOPENSUSE15)
 #define newlink(a,b,c,d,e) newlink(a,b,c,d)
 #endif
 
@@ -441,7 +467,8 @@ static inline struct nlattr **genl_family_attrbuf(const struct genl_family *fami
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0)
 #include <net/genetlink.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 13, 0) && !defined(ISRHEL7)
-#define genl_register_family(a) genl_register_family_with_ops(a, (struct genl_ops *)genl_ops, ARRAY_SIZE(genl_ops))
+#define genl_register_family(a) genl_register_family_with_ops(a, genl_ops, ARRAY_SIZE(genl_ops))
+#define COMPAT_CANNOT_USE_CONST_GENL_OPS
 #else
 #define genl_register_family(a) genl_register_family_with_ops(a, genl_ops)
 #endif
@@ -454,7 +481,7 @@ static inline struct nlattr **genl_family_attrbuf(const struct genl_family *fami
 #else
 #define ___COMPAT_NETLINK_DUMP_BLOCK return get_device_dump_real(skb, cb);
 #endif
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 14) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)) || LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 63)
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 15, 8) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0)) || (LINUX_VERSION_CODE < KERNEL_VERSION(4, 14, 25) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)) || LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 87)
 #define get_device_dump(a, b) get_device_dump_real(a, b); \
 static int get_device_dump(a, b) { \
 	struct wireguard_device *wg = (struct wireguard_device *)cb->args[0]; \
@@ -503,8 +530,11 @@ static inline int cpu_has_xfeatures(u64 xfeatures_needed, const char **feature_n
 #ifndef XFEATURE_MASK_SSE
 #define XFEATURE_MASK_SSE XSTATE_SSE
 #endif
-#ifndef XFEATURE_MASK_ZMM_Hi256
-#define XFEATURE_MASK_ZMM_Hi256 XSTATE_ZMM_Hi256
+#ifndef XSTATE_AVX512
+#define XSTATE_AVX512 (XSTATE_OPMASK | XSTATE_ZMM_Hi256 | XSTATE_Hi16_ZMM)
+#endif
+#ifndef XFEATURE_MASK_AVX512
+#define XFEATURE_MASK_AVX512 XSTATE_AVX512
 #endif
 #endif
 
